@@ -381,10 +381,10 @@ void Map_Mutations(t_node *a, t_node *d, int sa, int sd, t_edge *b, int site, in
   
   /* PhyML_Printf("\n. Mutmap: a:%d d:%d ta:%G td:%G cr:%G rr:%G l:%G", */
   /*              a?a->num:-1,d?d->num:-1, */
-  /*              tree->rates->nd_t[a->num], */
-  /*              tree->rates->nd_t[d->num], */
+  /*              tree->times->nd_t[a->num], */
+  /*              tree->times->nd_t[d->num], */
   /*              cr,rr, */
-  /*              fabs(tree->rates->nd_t[a->num]-tree->rates->nd_t[d->num])*cr*rr); */
+  /*              fabs(tree->times->nd_t[a->num]-tree->times->nd_t[d->num])*cr*rr); */
 
   // Matrix of change probabilities
   for(i=0;i<ns;i++)
@@ -461,7 +461,7 @@ void Map_Mutations(t_node *a, t_node *d, int sa, int sd, t_edge *b, int site, in
 #ifdef PHYTIME
           // Transform into time in calendar units
           muttime[(*n_mut)+n_mut_branch-1] /= tree->rates->cur_l[d->num];
-          muttime[(*n_mut)+n_mut_branch-1] *= fabs(tree->rates->nd_t[a->num]-tree->rates->nd_t[d->num]);
+          muttime[(*n_mut)+n_mut_branch-1] *= fabs(tree->times->nd_t[a->num]-tree->times->nd_t[d->num]);
 #endif
 
           tax_idx = -1;
@@ -562,10 +562,15 @@ void Ancestral_Sequences(t_tree *tree, int print)
       PhyML_Fprintf(tree->io->fp_out_ancestral_seq,"\n. found in the file '%s'.",tree->io->out_ancestral_tree_file);
       PhyML_Fprintf(tree->io->fp_out_ancestral_seq,"\n");
       PhyML_Fprintf(tree->io->fp_out_ancestral_seq,"\n");
-      PhyML_Fprintf(tree->io->fp_out_ancestral_seq,"\n. Recommended citation when using the MPEE criterion:");
+      PhyML_Fprintf(tree->io->fp_out_ancestral_seq,"\n. Ancestral reconstruction is conducted based on the \"Minimum Posterior Expected");
+      PhyML_Fprintf(tree->io->fp_out_ancestral_seq,"\n. Error\" (MPEE) criterion, which accomodates for uncertainty in the selection of ");
+      PhyML_Fprintf(tree->io->fp_out_ancestral_seq,"\n. ancestral character states.");
+      PhyML_Fprintf(tree->io->fp_out_ancestral_seq,"\n");
+      PhyML_Fprintf(tree->io->fp_out_ancestral_seq,"\n");
+      PhyML_Fprintf(tree->io->fp_out_ancestral_seq,"\n. Recommended citation:");
       PhyML_Fprintf(tree->io->fp_out_ancestral_seq,"\n. Oliva A., Pulicani S., Lefort V., Brehelin L., Gascuel O. and S. Guindon,");
       PhyML_Fprintf(tree->io->fp_out_ancestral_seq,"\n. \"Accounting for ambiguity in ancestral sequence reconstruction\",");
-      PhyML_Fprintf(tree->io->fp_out_ancestral_seq,"\n. Bioinformatics, 2019.");
+      PhyML_Fprintf(tree->io->fp_out_ancestral_seq,"\n. Bioinformatics, Volume 35, Issue 21, 1 November 2019.");
       PhyML_Fprintf(tree->io->fp_out_ancestral_seq,"\n");
       
       PhyML_Fprintf(tree->io->fp_out_ancestral_seq,"\n\n");
@@ -579,7 +584,7 @@ void Ancestral_Sequences(t_tree *tree, int print)
       
       tree->io->print_node_num = YES;
       tree->io->print_support_val = NO;
-      char *s = Write_Tree(tree,NO);
+      char *s = Write_Tree(tree);
       PhyML_Fprintf(tree->io->fp_out_ancestral_tree,"%s",s);
       tree->io->print_node_num = NO;
       tree->io->print_support_val = bck_support;
@@ -835,35 +840,58 @@ void Ancestral_Sequences_One_Node(t_node *d, t_tree *tree, int print)
 
                 }
 
+              if(tree->mod->ras->invar == YES)
+                {
+                  int num_prec_issue = NO;
+                  phydbl inv_site_lk = Invariant_Lk(sum_scale,csite,&num_prec_issue,tree);
+
+
+                  switch(num_prec_issue)
+                    {
+                    case YES :         
+                      {
+                        assert(isinf(inv_site_lk));
+                        inv_site_lk = Invariant_Lk(0,csite,&num_prec_issue,tree);
+                        for(i=0;i<ns;i++) p[i] = inv_site_lk * tree->mod->ras->pinvar->v * tree->mod->e_frq->pi->v[i];
+                        break;
+                      }
+                    case NO : 
+                      {
+                        for(i=0;i<ns;i++) p[i] = p[i] * (1.-tree->mod->ras->pinvar->v) + inv_site_lk * tree->mod->ras->pinvar->v * tree->mod->e_frq->pi->v[i];
+                        break;
+                      }
+                    }
+
+                }
+
+              
               for(i=0;i<ns;i++) p[i] = log(p[i]) - (phydbl)LOG2 * sum_scale;
               for(i=0;i<ns;i++) p[i] -= tree->c_lnL_sorted[csite];
               for(i=0;i<ns;i++) p[i] = exp(p[i]);
+
+
               
               /* sum_probas = 0.0; */
               /* for(i=0;i<ns;i++) sum_probas += p[i]; */
               /* for(i=0;i<ns;i++) p[i]/=sum_probas; */
               
+              sum_probas = 0.0;
+              for(i=0;i<ns;i++) sum_probas += p[i];
+              if(Are_Equal(sum_probas,1.0,0.01) == NO)
+                {
+                  PhyML_Fprintf(stderr,"\n. Probabilities do not sum to 1.0! Aborting.");
+                  for(i=0;i<ns;++i) PhyML_Fprintf(stderr,"\n. p[%2d]=%G",i,p[i]);
+                  Exit("\n");
+                }
               
               if(print == YES)
                 {
-                  PhyML_Fprintf(fp,"%4d\t%9d\t",site+1,d->num);
-                  sum_probas = .0;
-                  for(i=0;i<ns;i++)
-                    {
-                      PhyML_Fprintf(fp,"%10g\t",p[i]);
-                      sum_probas += p[i];
-                    }
-                  if(Are_Equal(sum_probas,1.0,0.01) == NO)
-                    {
-                      PhyML_Fprintf(stderr,"\n. Probabilities do not sum to 1.0! Aborting.");
-                      for(i=0;i<ns;++i) PhyML_Fprintf(stderr,"\n. p[%2d]=%G",i,p[i]);
-                      Exit("\n");
-                    }
-
+                  PhyML_Fprintf(fp,"%4d\t%9d\t",site+1,d->num);                  
+                  for(i=0;i<ns;i++) PhyML_Fprintf(fp,"%10g\t",p[i]);
                   PhyML_Fprintf(fp,"%s",Bit_To_Character_String(Integer_To_Bit(MPEE_Infer(p,ns),ns),ns));
-
                   PhyML_Fprintf(fp,"\n");
                 }
+
               
               
             }
