@@ -127,7 +127,7 @@ void DATE_XML(char *xml_filename)
     {
       PhyML_Fprintf(stdout,"\n. The model of rate variation across lineages is not specified.");
       PhyML_Fprintf(stdout,"\n. Using the geometric Brownian model (see Guindon, 2012, Syst. Biol.).\n");
-      mixt_tree->rates->model      = GUINDON;
+      mixt_tree->rates->model_id = GUINDON;
       mixt_tree->mod->gamma_mgf_bl = YES;
       strcpy(mixt_tree->rates->model_name,"geometric Brownian"); 
     }
@@ -147,52 +147,70 @@ void DATE_XML(char *xml_filename)
         {
           if(!strcmp(model_name,"geometricbrownian"))
             {
-              mixt_tree->rates->model      = GUINDON;
+              mixt_tree->rates->model_id = GUINDON;
               mixt_tree->mod->gamma_mgf_bl = YES;
-              strcpy(mixt_tree->rates->model_name,"geometric Brownian"); 
+              strcpy(mixt_tree->rates->model_name,"integrated"); 
             }
           else if(!strcmp(model_name,"geometric"))
             {
-              mixt_tree->rates->model      = GUINDON;
+              mixt_tree->rates->model_id = GUINDON;
               mixt_tree->mod->gamma_mgf_bl = YES;
-              strcpy(mixt_tree->rates->model_name,"geometric Brownian"); 
+              strcpy(mixt_tree->rates->model_name,"integrated"); 
             }
-          else if(!strcmp(model_name,"brownian"))
+          else if(!strcmp(model_name,"integrated"))
             {
-              mixt_tree->rates->model      = GUINDON;
+              mixt_tree->rates->model_id = GUINDON;
               mixt_tree->mod->gamma_mgf_bl = YES;
-              strcpy(mixt_tree->rates->model_name,"geometric Brownian"); 
+              strcpy(mixt_tree->rates->model_name,"integrated"); 
             }
           else if(!strcmp(model_name,"geo"))
             {
-              mixt_tree->rates->model      = GUINDON;
+              mixt_tree->rates->model_id = GUINDON;
               mixt_tree->mod->gamma_mgf_bl = YES;
-              strcpy(mixt_tree->rates->model_name,"geometric Brownian"); 
+              strcpy(mixt_tree->rates->model_name,"integrated"); 
             }
           else if(!strcmp(model_name,"lognormal"))
             {
-              mixt_tree->rates->model      = LOGNORMAL;
+              mixt_tree->rates->model_id = LOGNORMAL;
               mixt_tree->mod->gamma_mgf_bl = NO;
               strcpy(mixt_tree->rates->model_name,"lognormal (uncorrelated)"); 
             }
           else if(!strcmp(model_name,"normal"))
             {
-              mixt_tree->rates->model      = LOGNORMAL;
+              mixt_tree->rates->model_id = LOGNORMAL;
               mixt_tree->mod->gamma_mgf_bl = NO;
               strcpy(mixt_tree->rates->model_name,"lognormal (uncorrelated)"); 
             }
           else if(!strcmp(model_name,"strictclock"))
             {
-              mixt_tree->rates->model      = STRICTCLOCK;
+              mixt_tree->rates->model_id = STRICTCLOCK;
               mixt_tree->mod->gamma_mgf_bl = NO;
               strcpy(mixt_tree->rates->model_name,"strict clock"); 
             }
           else if(!strcmp(model_name,"clock"))
             {
-              mixt_tree->rates->model      = STRICTCLOCK;
+              mixt_tree->rates->model_id = STRICTCLOCK;
               mixt_tree->mod->gamma_mgf_bl = NO;
               strcpy(mixt_tree->rates->model_name,"strict clock"); 
             }
+          else if(!strcmp(model_name,"thorne"))
+            {
+              mixt_tree->rates->model_id = THORNE;
+              mixt_tree->mod->gamma_mgf_bl = NO;
+              strcpy(mixt_tree->rates->model_name,"autocorrelated"); 
+            }
+          else if(!strcmp(model_name,"autocorrelated"))
+            {
+              mixt_tree->rates->model_id = THORNE;
+              mixt_tree->mod->gamma_mgf_bl = NO;
+              strcpy(mixt_tree->rates->model_name,"autocorrelated"); 
+            }
+          else if(!strcmp(model_name,"autocorr"))
+            {
+              mixt_tree->rates->model_id = THORNE;
+              mixt_tree->mod->gamma_mgf_bl = NO;
+              strcpy(mixt_tree->rates->model_name,"autocorrelated"); 
+            }
           else
             {
               assert(FALSE);
@@ -201,6 +219,41 @@ void DATE_XML(char *xml_filename)
     }
   
   
+  // Looking for XML node with rate-across-lineage info
+  xnd = XML_Search_Node_Name("clockrate",YES,xroot);
+  
+  if(xnd != NULL)
+    {
+      char *clock_r;
+      clock_r = XML_Get_Attribute_Value(xnd,"value");
+      if(clock_r == NULL) clock_r = XML_Get_Attribute_Value(xnd,"clock.val");
+      if(clock_r == NULL) clock_r = XML_Get_Attribute_Value(xnd,"val");
+      if(clock_r != NULL)
+        {
+          mixt_tree->rates->clock_r = String_To_Dbl(clock_r);
+          for(int i=0;i<2*mixt_tree->n_otu-1;++i)
+            {
+              mixt_tree->rates->br_r[i] = mixt_tree->rates->clock_r;
+              mixt_tree->rates->nd_r[i] = mixt_tree->rates->clock_r;
+            }
+        }
+      
+      char *opt_clock;
+      opt_clock = XML_Get_Attribute_Value(xnd,"optimise.clock");
+      if(opt_clock == NULL) opt_clock = XML_Get_Attribute_Value(xnd,"optimize.clock");
+      if(opt_clock == NULL) opt_clock = XML_Get_Attribute_Value(xnd,"optimize.rate");
+      if(opt_clock == NULL) opt_clock = XML_Get_Attribute_Value(xnd,"opt.clock");
+      
+      if(opt_clock != NULL)
+        {
+          int select = XML_Validate_Attr_Int(opt_clock,6,
+                                             "true","yes","y",
+                                             "false","no","n");
+          if(select < 3) mixt_tree->mod->s_opt->opt_clock_r = YES;
+          else mixt_tree->mod->s_opt->opt_clock_r = NO;
+        }
+    }
+
   // Looking for calibration info
   xnd = XML_Search_Node_Name("calibration",YES,xroot);
 
@@ -253,17 +306,20 @@ void DATE_XML(char *xml_filename)
 
 
   MIXT_Check_Model_Validity(mixt_tree);
-  MIXT_Init_Model(mixt_tree);
+  MIXT_Chain_Models(mixt_tree);
+  Init_Model(mixt_tree->mod->io->cdata,mixt_tree->mod,mixt_tree->mod->io);
+  Set_Model_Parameters(mixt_tree->mod);
   Print_Data_Structure(NO,stdout,mixt_tree);
   tree = MIXT_Starting_Tree(mixt_tree);
   Add_Root(tree->a_edges[0],tree);
   Copy_Tree(tree,mixt_tree);
   Free_Tree(tree);
-  MIXT_Connect_Cseqs_To_Nodes(mixt_tree);
-  MIXT_Init_T_Beg(mixt_tree);  
-  MIXT_Make_Tree_For_Lk(mixt_tree);
-  MIXT_Make_Tree_For_Pars(mixt_tree);
-  MIXT_Make_Spr(mixt_tree);  
+  Copy_Tree(mixt_tree,mixt_tree->next);
+  Connect_CSeqs_To_Nodes(mixt_tree->mod->io->cdata,mixt_tree->mod->io,mixt_tree);
+  Init_T_Beg(mixt_tree);  
+  Make_Tree_For_Lk(mixt_tree);
+  Make_Tree_For_Pars(mixt_tree);
+  Make_Spr(mixt_tree);  
   MIXT_Chain_All(mixt_tree);
   MIXT_Check_Edge_Lens_In_All_Elem(mixt_tree);
   MIXT_Turn_Branches_OnOff_In_All_Elem(ON,mixt_tree);
@@ -278,11 +334,11 @@ void DATE_XML(char *xml_filename)
 
   // Cleaning up...
   RATES_Free_Rates(mixt_tree->rates);
-  RATES_Free_Rates(mixt_tree->extra_tree->rates);
+  RATES_Free_Rates(mixt_tree->aux_tree[0]->rates);
   TIMES_Free_Times(mixt_tree->times);
-  TIMES_Free_Times(mixt_tree->extra_tree->times);
+  TIMES_Free_Times(mixt_tree->aux_tree[0]->times);
   MCMC_Free_MCMC(mixt_tree->mcmc);
-  MCMC_Free_MCMC(mixt_tree->extra_tree->mcmc);
+  MCMC_Free_MCMC(mixt_tree->aux_tree[0]->mcmc);
   Free_Mmod(mixt_tree->mmod);
   Free_Spr_List_One_Edge(mixt_tree);
   Free_Tree_Pars(mixt_tree);
@@ -314,7 +370,8 @@ void DATE_XML(char *xml_filename)
   
   Free_Model_Complete(mixt_tree->mod);
   Free_Model_Basic(mixt_tree->mod);
-  Free_Tree(mixt_tree->extra_tree);  
+  Free_Tree(mixt_tree->aux_tree[0]);  
+  Free(mixt_tree->aux_tree);  
   Free_Tree(mixt_tree);  
   Free(res);
   XML_Free_XML_Tree(xroot);
@@ -764,15 +821,15 @@ phydbl *DATE_MCMC(t_tree *tree)
   Set_Update_Eigen(NO,tree->mod);
 
   
-  RATES_Lk_Rates(tree);
+  RATES_Lk(tree);
   DATE_Assign_Primary_Calibration(tree);
-  TIMES_Lk_Times(NO,tree);
+  TIMES_Lk(tree);
 
   /* Time_To_Branch(tree); */
   /* tree->bl_ndigits = 1; */
   /* printf("\n. Random init tree: %s",Write_Tree(tree)); */
   /* tree->bl_ndigits = 7; */
-  RATES_Update_Cur_Bl(tree);
+  RATES_Update_Edge_Lengths(tree);
 
       
   PhyML_Printf("\n. AVX enabled: %s",
@@ -794,34 +851,35 @@ phydbl *DATE_MCMC(t_tree *tree)
   PhyML_Printf("\n. Ignore sequences: %s",tree->eval_alnL == YES ? "no" : "yes");
   PhyML_Printf("\n. Model of variation of rates across lineages: %s",tree->rates->model_name);
   PhyML_Printf("\n. log(Pr(Seq|Tree)) = %f",tree->c_lnL);
-  PhyML_Printf("\n. log(Pr(Tree)) = %f",tree->times->c_lnL_times);
+  PhyML_Printf("\n. log(Pr(Tree)) = %f",tree->times->c_lnL);
     
-    
-  tree->extra_tree = Make_Tree_From_Scratch(tree->n_otu,tree->data);
-  tree->extra_tree->mod = tree->mod;
-  Copy_Tree(tree,tree->extra_tree);
-
-  tree->extra_tree->rates = RATES_Make_Rate_Struct(tree->n_otu);
-  RATES_Init_Rate_Struct(tree->extra_tree->rates,NULL,tree->n_otu);
-  RATES_Copy_Rate_Struct(tree->rates,tree->extra_tree->rates,tree->n_otu);
-  tree->extra_tree->rates->model = LOGNORMAL;
-
-  tree->extra_tree->times = TIMES_Make_Time_Struct(tree->n_otu);
-  TIMES_Init_Time_Struct(tree->extra_tree->times,NULL,tree->n_otu);
-  TIMES_Copy_Time_Struct(tree->times,tree->extra_tree->times,tree->n_otu);
-
-  RATES_Duplicate_Calib_Struct(tree,tree->extra_tree);
-  MIXT_Chain_Cal(tree->extra_tree);  
-  DATE_Assign_Primary_Calibration(tree->extra_tree);
-  TIMES_Randomize_Tree_With_Time_Constraints(tree->extra_tree->times->a_cal[0],tree->extra_tree);
   
+  tree->aux_tree = (t_tree **)mCalloc(1,sizeof(t_tree *));
+
+  tree->aux_tree[0] = Make_Tree_From_Scratch(tree->n_otu,tree->data);
+  tree->aux_tree[0]->mod = tree->mod;
+  Copy_Tree(tree,tree->aux_tree[0]);
+
+  tree->aux_tree[0]->rates = RATES_Make_Rate_Struct(tree->n_otu);
+  RATES_Init_Rate_Struct(tree->aux_tree[0]->rates,NULL,tree->n_otu);
+  RATES_Copy_Rate_Struct(tree->rates,tree->aux_tree[0]->rates,tree->n_otu);
+  tree->aux_tree[0]->rates->model_id = LOGNORMAL;
+
+  tree->aux_tree[0]->times = TIMES_Make_Time_Struct(tree->n_otu);
+  TIMES_Init_Time_Struct(tree->aux_tree[0]->times,NULL,tree->n_otu);
+  TIMES_Copy_Time_Struct(tree->times,tree->aux_tree[0]->times,tree->n_otu);
+
+  RATES_Duplicate_Calib_Struct(tree,tree->aux_tree[0]);
+  MIXT_Chain_Cal(tree->aux_tree[0]);  
+  DATE_Assign_Primary_Calibration(tree->aux_tree[0]);
+  TIMES_Randomize_Tree_With_Time_Constraints(tree->aux_tree[0]->times->a_cal[0],tree->aux_tree[0]);
   
-  TIMES_Lk_Times(NO,tree->extra_tree);
-  PhyML_Printf("\n. log(Pr(extra tree)) = %f",tree->extra_tree->times->c_lnL_times);
+  TIMES_Lk(tree->aux_tree[0]);
+  PhyML_Printf("\n. log(Pr(extra tree)) = %f",tree->aux_tree[0]->times->c_lnL);
   mcmc = MCMC_Make_MCMC_Struct();
-  tree->extra_tree->mcmc = mcmc;
+  tree->aux_tree[0]->mcmc = mcmc;
   MCMC_Init_MCMC_Struct(NULL,NULL,mcmc);
-  MCMC_Complete_MCMC(mcmc,tree->extra_tree);
+  MCMC_Complete_MCMC(mcmc,tree->aux_tree[0]);
       
   PhyML_Fprintf(fp_stats,"\n%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t",
                 "sample",
@@ -856,9 +914,9 @@ phydbl *DATE_MCMC(t_tree *tree)
     }
 
   
-  if(tree->rates->model == THORNE ||
-     tree->rates->model == LOGNORMAL ||
-     tree->rates->model == STRICTCLOCK)
+  if(tree->rates->model_id == THORNE ||
+     tree->rates->model_id == LOGNORMAL ||
+     tree->rates->model_id == STRICTCLOCK)
     for(i=0;i<2*tree->n_otu-2;++i) PhyML_Fprintf(fp_stats,"br%d\t",i);
   else
     for(i=0;i<2*tree->n_otu-1;++i) PhyML_Fprintf(fp_stats,"nr%d\t",i);
@@ -972,10 +1030,10 @@ phydbl *DATE_MCMC(t_tree *tree)
         }
       
       
-      if(!(tree->times->c_lnL_times > UNLIKELY))
+      if(!(tree->times->c_lnL > UNLIKELY))
         {
           PhyML_Fprintf(stderr,"\n. move: %s",tree->mcmc->move_name[move]);
-          PhyML_Fprintf(stderr,"\n. glnL=%f",tree->times->c_lnL_times);
+          PhyML_Fprintf(stderr,"\n. glnL=%f",tree->times->c_lnL);
           assert(FALSE);
         }
 
@@ -986,7 +1044,7 @@ phydbl *DATE_MCMC(t_tree *tree)
           phydbl mean_r,post;
 
           mean_r = RATES_Average_Substitution_Rate(tree);
-          post = Get_Lk(tree) + tree->times->c_lnL_times + tree->rates->c_lnL_rates;
+          post = Get_Lk(tree) + tree->times->c_lnL + tree->rates->c_lnL;
 
           if(tree->mcmc->run < adjust_len) PhyML_Printf("\nx");
           else PhyML_Printf("\n.");
@@ -994,8 +1052,8 @@ phydbl *DATE_MCMC(t_tree *tree)
                        tree->mcmc->run,
                        post,
                        Get_Lk(tree),
-                       tree->times->c_lnL_times,
-                       tree->rates->c_lnL_rates,
+                       tree->times->c_lnL,
+                       tree->rates->c_lnL,
                        fabs(tree->times->nd_t[tree->n_root->num]),                       
                        (int)time(NULL) - t_beg,
                        mean_r,
@@ -1007,14 +1065,14 @@ phydbl *DATE_MCMC(t_tree *tree)
           phydbl mean_r,post;
           
           mean_r = RATES_Average_Substitution_Rate(tree);
-          post = Get_Lk(tree) + tree->times->c_lnL_times + tree->rates->c_lnL_rates;
+          post = Get_Lk(tree) + tree->times->c_lnL + tree->rates->c_lnL;
           
           PhyML_Fprintf(fp_stats,"\n%6d\t%9.1f\t%9.1f\t%9.1f\t%9.1f\t%12G\t%12G\t%12G\t%12G\t%12G\t%12G\t",
                         tree->mcmc->run,
                         post,
                         Get_Lk(tree),
-                        tree->times->c_lnL_times,
-                        tree->rates->c_lnL_rates,
+                        tree->times->c_lnL,
+                        tree->rates->c_lnL,
                         tree->times->birth_rate,
                         tree->times->death_rate,
                         mean_r,
@@ -1043,9 +1101,9 @@ phydbl *DATE_MCMC(t_tree *tree)
               /* PhyML_Fprintf(fp_stats,"%s\t",cal->clade_list[cal->current_clade_idx]->id); */
             }
 
-          if(tree->rates->model == THORNE ||
-             tree->rates->model == LOGNORMAL ||
-             tree->rates->model == STRICTCLOCK)
+          if(tree->rates->model_id == THORNE ||
+             tree->rates->model_id == LOGNORMAL ||
+             tree->rates->model_id == STRICTCLOCK)
             for(i=0;i<2*tree->n_otu-2;++i) PhyML_Fprintf(fp_stats,"%G\t",tree->rates->br_r[i]);
           else
             for(i=0;i<2*tree->n_otu-1;++i) PhyML_Fprintf(fp_stats,"%G\t",tree->rates->nd_r[i]);
@@ -1089,14 +1147,14 @@ phydbl *DATE_MCMC(t_tree *tree)
           
           TIMES_Time_To_Bl(tree);
           tree->bl_ndigits = 3;
-          /* RATES_Update_Cur_Bl(tree); */
+          /* RATES_Update_Edge_Lengths(tree); */
           s_tree = Write_Tree(tree);
           tree->bl_ndigits = 7;
           PhyML_Fprintf(fp_tree,"\ntree %d [&lnP=%f] = [&R] %s",tree->mcmc->sample_num,tree->c_lnL,s_tree);          
           Free(s_tree);
           PhyML_Fprintf(fp_tree,"\nEND;");          
           fflush(NULL);
-          RATES_Update_Cur_Bl(tree);
+          RATES_Update_Edge_Lengths(tree);
 
           
           if(tree->mcmc->run > tree->mcmc->chain_len_burnin &&
