@@ -75,8 +75,9 @@ void Free_Edge_Lk_Left(t_edge *b);
 void Free_Edge_Lk_Rght(t_edge *b);
 void Free_Geo(t_geo *t);
 void Free_Geo_Coord(t_geo_coord *t);
-void Free_Disk(t_dsk *t);
-void Free_Ldisk(t_ldsk *t);
+void PHYREX_Free_Disk(t_dsk *t);
+void PHYREX_Free_Ldisk(t_ldsk *t);
+void PHYREX_Free_Ldsk_Struct(t_tree *tree);
 void Free_Poly(t_poly *p);
 void Free_Mmod(t_phyrex_mod *mmod);
 void Free_Efrq_Weights(t_mod *mixt_mod);
@@ -98,5 +99,8 @@ void Free_Clade(t_clad *this);
 void Free_Label(t_label *lab);
 void Free_Contrasts(t_ctrst *ctrst);
 void TIMES_Free_Times(t_time *times);
+void Free_Best_Spr(t_tree *tree);
+void Free_All_Edges_Light(t_tree *tree);
+void Free_All_Edges_Lens(t_tree *tree);
 
 #endif
