@@ -27,7 +27,6 @@ void PMat_JC69(phydbl l, int pos, phydbl *Pij, t_mod *mod)
 
   ns = mod->ns;
 
-
   for(i=0;i<ns;i++) Pij[pos+ ns*i+i] = 1. - ((ns - 1.)/ns)*(1. - exp(-ns*l/(ns - 1.)));
   for(i=0;i<ns-1;i++)
     for(j=i+1;j<ns;j++)
@@ -422,7 +421,7 @@ void PMat(phydbl l, t_mod *mod, int pos, phydbl *Pij, phydbl *tPij)
 #ifdef BEAGLE
       //when there is no active instance (i.e. when we are building the initial tree)
       if(UNINITIALIZED == mod->b_inst) PMat_Empirical(l,mod,pos,Pij,tPij);
-#else
+#else      
       PMat_Empirical(l,mod,pos,Pij,tPij);
 #endif
     }
@@ -431,7 +430,6 @@ void PMat(phydbl l, t_mod *mod, int pos, phydbl *Pij, phydbl *tPij)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-
 int GetDaa (phydbl *daa, phydbl *pi, char *file_name)
 {
 /* Get the amino acid distance (or substitution rate) matrix
@@ -481,30 +479,23 @@ int GetDaa (phydbl *daa, phydbl *pi, char *file_name)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-
-void Update_Qmat_Generic(phydbl *rr, phydbl *pi, int ns, phydbl *qmat)
+phydbl Update_Qmat_Generic(phydbl *rr, phydbl *pi, int ns, phydbl *qmat)
 {
   int i,j;
   phydbl sum,mr;
 
   for(i=0;i<ns*ns;++i) qmat[i] = .0;
 
-  if(rr[(int)(ns*(ns-1)/2)-1] < 0.00001)
-    {
-      PhyML_Fprintf(stderr,"\n. rr[%d]=%f",(int)(ns*(ns-1)/2)-1,rr[(int)(ns*(ns-1)/2)-1]);
-      assert(FALSE);
-    }
-  
-  /* PhyML_Printf("\n"); */
-  /* For(i,(int)(ns*(ns-1)/2)) */
+  /* if(rr[(int)(ns*(ns-1)/2)-1] < 0.00001) */
   /*   { */
-  /*     PhyML_Printf("\n> rr %d = %f",i,rr[i]); */
+  /*     PhyML_Fprintf(stderr,"\n. rr[%d]=%f",(int)(ns*(ns-1)/2)-1,rr[(int)(ns*(ns-1)/2)-1]); */
+  /*     assert(FALSE); */
+  /*   } */
+  
+  /* for(i=0;i<(int)(ns*(ns-1)/2);++i) */
+  /*   { */
+  /*     rr[i] /= rr[(int)(ns*(ns-1)/2)-1]; */
   /*   } */
-
-  for(i=0;i<(int)(ns*(ns-1)/2);++i)
-    {
-      rr[i] /= rr[(int)(ns*(ns-1)/2)-1];
-    }
   
   /* Fill the non-diagonal parts */
   for(i=0;i<ns;i++)
@@ -536,7 +527,9 @@ void Update_Qmat_Generic(phydbl *rr, phydbl *pi, int ns, phydbl *qmat)
       mr += sum * pi[i];
     }
 
-  /* for(i=0;i<ns;i++) for(j=0;j<ns;j++) qmat[i*ns+j] /= mr; */
+  for(i=0;i<ns*ns;i++) qmat[i] /= mr;
+
+  return(mr);
 }
 
 //////////////////////////////////////////////////////////////
@@ -548,6 +541,7 @@ void Update_Qmat_GTR(phydbl *rr, phydbl *rr_val, int *rr_num, phydbl *pi, phydbl
   phydbl mr;
   
   for(i=0;i<6;i++) rr[i] = exp(rr_val[rr_num[i]]);
+  /* for(i=0;i<6;i++) rr[i] = log(rr_val[rr_num[i]]); */
   for(i=0;i<6;i++)
     if(rr[i] < 0.0)
       {
@@ -597,7 +591,6 @@ void Update_Qmat_GTR(phydbl *rr, phydbl *rr_val, int *rr_num, phydbl *pi, phydbl
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-
 void Update_Qmat_HKY(phydbl kappa, phydbl *pi, phydbl *qmat)
 {
   int i;
@@ -626,7 +619,42 @@ void Update_Qmat_HKY(phydbl kappa, phydbl *pi, phydbl *qmat)
 
   /* compute diagonal terms of Q and mean rate mr = l/t */
   mr = .0;
-  For (i,4) mr += pi[i] * (-qmat[i*4+i]);
+  for(i=0;i<4;++i) mr += pi[i] * (-qmat[i*4+i]);
+  for(i=0;i<16;i++) qmat[i] /= mr;
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void Update_Qmat_TN93(phydbl kappa, phydbl lambda, phydbl *pi, phydbl *qmat)
+{
+  int i;
+  phydbl mr;
+
+  /* A -> C */ qmat[0*4+1] = (phydbl)(pi[1]);
+  /* A -> G */ qmat[0*4+2] = (phydbl)(kappa*lambda*pi[2]);
+  /* A -> T */ qmat[0*4+3] = (phydbl)(pi[3]);
+
+  /* C -> A */ qmat[1*4+0] = (phydbl)(pi[0]);
+  /* C -> G */ qmat[1*4+2] = (phydbl)(pi[2]);
+  /* C -> T */ qmat[1*4+3] = (phydbl)(kappa*pi[3]);
+
+  /* G -> A */ qmat[2*4+0] = (phydbl)(kappa*lambda*pi[0]);
+  /* G -> C */ qmat[2*4+1] = (phydbl)(pi[1]);
+  /* G -> T */ qmat[2*4+3] = (phydbl)(pi[3]);
+
+  /* T -> A */ qmat[3*4+0] = (phydbl)(pi[0]);
+  /* T -> C */ qmat[3*4+1] = (phydbl)(kappa*pi[1]);
+  /* T -> G */ qmat[3*4+2] = (phydbl)(pi[2]);
+
+  qmat[0*4+0] = (phydbl)(-(qmat[0*4+1]+qmat[0*4+2]+qmat[0*4+3]));
+  qmat[1*4+1] = (phydbl)(-(qmat[1*4+0]+qmat[1*4+2]+qmat[1*4+3]));
+  qmat[2*4+2] = (phydbl)(-(qmat[2*4+0]+qmat[2*4+1]+qmat[2*4+3]));
+  qmat[3*4+3] = (phydbl)(-(qmat[3*4+0]+qmat[3*4+1]+qmat[3*4+2]));
+
+  /* compute diagonal terms of Q and mean rate mr = l/t */
+  mr = .0;
+  for (i=0;i<4;++i) mr += pi[i] * (-qmat[i*4+i]);
   for(i=0;i<16;i++) qmat[i] /= mr;
 }
 
@@ -693,8 +721,7 @@ int Update_RAS(t_mod *mod)
       // Update class frequencies
 #ifndef PHYML
       Qksort(mod->ras->gamma_r_proba_unscaled->v,NULL,0,mod->ras->n_catg-1); // Unscaled class frequencies sorted in increasing order
-#endif
-      
+
       mod->ras->gamma_r_proba->v[0] = mod->ras->gamma_r_proba_unscaled->v[0] / mod->ras->gamma_r_proba_unscaled->v[mod->ras->n_catg-1];
       for(i=1;i<mod->ras->n_catg;i++)
         {
@@ -703,37 +730,35 @@ int Update_RAS(t_mod *mod)
              mod->ras->gamma_r_proba_unscaled->v[i-1]) /
             mod->ras->gamma_r_proba_unscaled->v[mod->ras->n_catg-1];
         }
-
-#ifndef PHYREX
-      do
-        {
-          sum = .0;
-          for(i=0;i<mod->ras->n_catg;i++)
-            {
-              if(mod->ras->gamma_r_proba->v[i] < 0.01) mod->ras->gamma_r_proba->v[i]=0.01;
-              if(mod->ras->gamma_r_proba->v[i] > 0.99) mod->ras->gamma_r_proba->v[i]=0.99;
-              sum += mod->ras->gamma_r_proba->v[i];
-            }
-          for(i=0;i<mod->ras->n_catg;i++) mod->ras->gamma_r_proba->v[i]/=sum;
-        }
-      while((sum > 1.01) || (sum < 0.99));
 #endif
-      
 
+#ifdef PHYML
+      sum = 0.0;
+      for(i=0;i<mod->ras->n_catg;i++) sum += exp(mod->ras->gamma_r_proba_unscaled->v[i]);
+      for(i=0;i<mod->ras->n_catg;i++) mod->ras->gamma_r_proba->v[i] = exp(mod->ras->gamma_r_proba_unscaled->v[i])/sum;
+#endif
+
+      
       // Update class rates
       if(mod->ras->normalise_rr == YES)
         {
+#ifdef PHYML
           sum = .0;
-          for(i=0;i<mod->ras->n_catg;i++) sum += mod->ras->gamma_r_proba->v[i] * mod->ras->gamma_rr_unscaled->v[i];
-          for(i=0;i<mod->ras->n_catg;i++) mod->ras->gamma_rr->v[i] = mod->ras->gamma_rr_unscaled->v[i]/sum;
+          for(i=0;i<mod->ras->n_catg;i++) sum += mod->ras->gamma_r_proba->v[i] * exp(mod->ras->gamma_rr_unscaled->v[i]);
+          for(i=0;i<mod->ras->n_catg;i++) mod->ras->gamma_rr->v[i] = exp(mod->ras->gamma_rr_unscaled->v[i])/sum;
+#else
+          sum = .0;
+          for(i=0;i<mod->ras->n_catg;i++) sum += mod->ras->gamma_r_proba->v[i] * fabs(mod->ras->gamma_rr_unscaled->v[i]);
+          for(i=0;i<mod->ras->n_catg;i++) mod->ras->gamma_rr->v[i] = fabs(mod->ras->gamma_rr_unscaled->v[i])/sum;
+#endif
         }
       else
         {
-          for(i=0;i<mod->ras->n_catg;i++) mod->ras->gamma_rr->v[i] = mod->ras->gamma_rr_unscaled->v[i] * mod->ras->free_rate_mr->v;
+          for(i=0;i<mod->ras->n_catg;i++) mod->ras->gamma_rr->v[i] = exp(mod->ras->gamma_rr_unscaled->v[i]);
         }
       
       /* printf("\n"); */
-      /* for(i=0;i<mod->ras->n_catg;i++)  */
+      /* for(i=0;i<mod->ras->n_catg;i++) */
       /*   printf("\nx %3d %12f %12f xx %12f %12f", */
       /*          mod->ras->normalise_rr, */
       /*          mod->ras->gamma_r_proba->v[i], */
@@ -745,6 +770,14 @@ int Update_RAS(t_mod *mod)
   if(UNINITIALIZED != mod->b_inst)
       update_beagle_ras(mod);
 #endif
+  
+  /* for(i=0;i<mod->ras->n_catg;i++) PhyML_Printf("\n. REALW%d: %12f [%12f] (%12f;%12f--%2d)", */
+  /*                                              i, */
+  /*                                              mod->ras->gamma_r_proba->v[i], */
+  /*                                              mod->ras->gamma_r_proba_unscaled->v[i], */
+  /*                                              mod->ras->gamma_rr->v[i], */
+  /*                                              mod->ras->gamma_rr_unscaled->v[i], */
+  /*                                              mod->ras->normalise_rr); */
 
   return 1;
 }
@@ -756,8 +789,8 @@ int Update_Efrq(t_mod *mod)
 {
   unsigned int i;
   phydbl sum;
-
-  if((mod->io->datatype == NT) && (mod->s_opt->opt_state_freq == YES))
+                                      
+  if(mod->s_opt->state_freq == ML)
     {
       for(i=0;i<mod->ns;++i) mod->e_frq->pi->v[i] = exp(mod->e_frq->pi_unscaled->v[i]);
       sum = 0.0;
@@ -769,10 +802,11 @@ int Update_Efrq(t_mod *mod)
         update_beagle_efrqs(mod);
 #endif
 
-      for(i=0;i<mod->ns;++i) if(mod->e_frq->pi->v[i] < E_FRQ_MIN) mod->e_frq->pi->v[i] = E_FRQ_MIN;
-      for(i=0;i<mod->ns;++i) if(mod->e_frq->pi->v[i] > E_FRQ_MAX) mod->e_frq->pi->v[i] = E_FRQ_MAX;
     }
 
+  for(i=0;i<mod->ns;++i) if(mod->e_frq->pi->v[i] < E_FRQ_MIN) mod->e_frq->pi->v[i] = E_FRQ_MIN;
+  for(i=0;i<mod->ns;++i) if(mod->e_frq->pi->v[i] > E_FRQ_MAX) mod->e_frq->pi->v[i] = E_FRQ_MAX;
+
   return 1;
 }
 
@@ -781,10 +815,10 @@ int Update_Efrq(t_mod *mod)
 
 int Set_Model_Parameters(t_mod *mod)
 {  
-  if(!Update_Boundaries(mod)) return 0;
   if(!Update_RAS(mod))        return 0;
   if(!Update_Efrq(mod))       return 0;
   if(!Update_Eigen(mod))      return 0;
+  if(!Update_Boundaries(mod)) return 0;
   if(mod->is_mixt_mod == YES) MIXT_Set_Model_Parameters(mod);
   return 1;
 }
@@ -875,20 +909,27 @@ int Update_Eigen(t_mod *mod)
             Update_Qmat_GTR(mod->r_mat->rr->v, mod->r_mat->rr_val->v, mod->r_mat->rr_num->v, mod->e_frq->pi->v, mod->r_mat->qmat->v, mod->s_opt->opt_rr);
           else if(mod->whichmodel == HKY85)
             Update_Qmat_HKY(mod->kappa->v, mod->e_frq->pi->v, mod->r_mat->qmat->v);
+          else if(mod->whichmodel == TN93)
+            Update_Qmat_TN93(mod->kappa->v, mod->lambda->v, mod->e_frq->pi->v, mod->r_mat->qmat->v);
           else /* Any other nucleotide-based t_mod */
             Update_Qmat_HKY(mod->kappa->v, mod->e_frq->pi->v, mod->r_mat->qmat->v);
         }
+      else if(mod->io->datatype == AA)
+        {
+          Update_Qmat_Generic(mod->r_mat->rr_val->v,mod->e_frq->pi->v,mod->ns,mod->r_mat->qmat->v);
+        }
       else if(mod->io->datatype == GENERIC)
         {
           Update_Qmat_Generic(mod->r_mat->rr_val->v,mod->e_frq->pi->v,mod->ns,mod->r_mat->qmat->v);
         }
+
+
       
       //Now compute the eigen vectors and values
       scalar   = 1.0;
       n_iter   = 0;
       result   = 0;
 
-      
       for(i=0;i<mod->ns*mod->ns;++i)
         {
           mod->r_mat->qmat_buff->v[i] = mod->r_mat->qmat->v[i];
@@ -934,7 +975,7 @@ int Update_Eigen(t_mod *mod)
                   Exit("\n");
                 }
 
-              For (i,mod->eigen->size*mod->eigen->size) mod->eigen->l_e_vect[i] = mod->eigen->r_e_vect[i];
+              for(i=0;i<mod->eigen->size*mod->eigen->size;++i) mod->eigen->l_e_vect[i] = mod->eigen->r_e_vect[i];
               n_iter++;
 	      if(n_iter > 100) 
                 {                  
