@@ -112,17 +112,19 @@ phydbl SLFV_Coalescence_Rate(t_tree *tree)
 
 /*////////////////////////////////////////////////////////////
 ////////////////////////////////////////////////////////////*/
-
-phydbl SLFV_Path_Logdensity(t_ldsk *young, t_ldsk *old, phydbl sd, t_tree *tree)
+// Log-density of path, conditionned on its length
+phydbl SLFV_Path_Logdensity(t_ldsk *young, t_ldsk *old, phydbl *sd, t_tree *tree)
 {
   int i,j,err;
   t_ldsk *ldsk;
-  phydbl lnDens,mode;
+  phydbl lnP,mode,var;
   phydbl X,Y,Xp,Yp;
-  phydbl slope,inter;
+  /* phydbl slope,inter,mode; */
   int dir_to_young;
+  phydbl dt_young,dt_old,sum;
+  phydbl pos;
   
-  lnDens = 0.0;
+  lnP = 0.0;
   mode   = 0.0;
     
   for(i=0;i<tree->mmod->n_dim;i++)
@@ -147,46 +149,54 @@ phydbl SLFV_Path_Logdensity(t_ldsk *young, t_ldsk *old, phydbl sd, t_tree *tree)
           Xp = ldsk->next[dir_to_young]->coord->lonlat[i];
           Yp = ldsk->next[dir_to_young]->disk->time;
           
-          slope = (Yp-Y)/(Xp-X);
-          inter = Y - slope*X;
+          /* slope = (Yp-Y)/(Xp-X); */
+          /* inter = Y - slope*X; */
+          
+          /* mode = (ldsk->disk->time - inter)/slope; */
 
+          dt_young = fabs(Yp - ldsk->disk->time);
+          dt_old = fabs(Y - ldsk->disk->time);
+          sum = dt_young + dt_old;
+          
+          mode = (dt_young/sum)*X + (dt_old/sum)*Xp;
+          var = dt_young*dt_old/sum * sd[i] * sd[i]; 
+          
           assert(ldsk != NULL);
-          
-          /* mode = (old->coord->lonlat[i] - young->coord->lonlat[i])/(n_evt+1.-j) + young->coord->lonlat[i]; */
-
-          mode = (ldsk->disk->time - inter)/slope;
-          
-          lnDens += Log_Dnorm_Trunc(ldsk->coord->lonlat[i],
+                    
+          lnP += Log_Dnorm_Trunc(ldsk->coord->lonlat[i],
                                     mode,
-                                    sd,
+                                    sqrt(var),
                                     tree->mmod->lim_do->lonlat[i],
                                     tree->mmod->lim_up->lonlat[i],&err);
 
-          /* lnDens += Log_Dnorm_Trunc(ldsk->disk->centr->lonlat[i], */
-          /*                           mode, */
-          /*                           sd, */
-          /*                           0.0, */
-          /*                           tree->mmod->lim->lonlat[i],&err); */
+          /* lnP += Log_Dnorm(ldsk->coord->lonlat[i], */
+          /*                     mode, */
+          /*                     sqrt(var), */
+          /*                     &err); */
 
-          lnDens += Log_Dnorm_Trunc(ldsk->disk->centr->lonlat[i],
-                                    ldsk->coord->lonlat[i],
-                                    sd,
-                                    tree->mmod->lim_do->lonlat[i],
-                                    tree->mmod->lim_up->lonlat[i],&err);
-          /* lnDens += log(1./tree->mmod->lim->lonlat[i]); */
+          pos = 0.0;
+          if(ldsk->coord->lonlat[i] > tree->mmod->lim_up->lonlat[i]) pos = tree->mmod->lim_up->lonlat[i];
+          else if(ldsk->coord->lonlat[i] < tree->mmod->lim_do->lonlat[i]) pos = tree->mmod->lim_do->lonlat[i];
+          else pos = ldsk->coord->lonlat[i];
+          
+          lnP += Log_Dnorm_Trunc(ldsk->disk->centr->lonlat[i],
+                                 pos,
+                                 sd[i],
+                                 tree->mmod->lim_do->lonlat[i],
+                                 tree->mmod->lim_up->lonlat[i],&err);
 
           ldsk = ldsk->prev;
           j++;
         }
     }
   
-  return(lnDens);
+  return(lnP);
 }
 
 /*////////////////////////////////////////////////////////////
 ////////////////////////////////////////////////////////////*/
 
-void SLFV_Sample_Path(t_ldsk *young, t_ldsk *old, phydbl sd, phydbl *global_hr, t_tree *tree)
+void SLFV_Sample_Path(t_ldsk *young, t_ldsk *old, phydbl *sd, phydbl *global_hr, t_tree *tree)
 {
   phydbl new_ldsk_pos,cur_ldsk_pos;
   phydbl new_centr_pos,cur_centr_pos;
@@ -197,7 +207,6 @@ void SLFV_Sample_Path(t_ldsk *young, t_ldsk *old, phydbl sd, phydbl *global_hr,
   t_ldsk *ldsk;
   int accept,reject;
   int path_len;
-  
 
   path_len = PHYREX_Path_Len(young,old)-2;
   if(path_len == 0) return;
@@ -208,8 +217,7 @@ void SLFV_Sample_Path(t_ldsk *young, t_ldsk *old, phydbl sd, phydbl *global_hr,
   n_mcmc_iter = 0;
   accept = reject = 0;
   do
-    {
-          
+    {          
       ldsk = young->prev;
       do
         {
@@ -220,8 +228,7 @@ void SLFV_Sample_Path(t_ldsk *young, t_ldsk *old, phydbl sd, phydbl *global_hr,
           
           hr = 0.0;
           ratio = 0.0;
-          
-          /* cur_glnL = SLFV_Lk_Range(young->disk,old->disk,tree); */
+
           cur_glnL = SLFV_Lk_Range(ldsk->disk,ldsk->prev->disk,tree);
           new_glnL = cur_glnL;
           
@@ -238,7 +245,7 @@ void SLFV_Sample_Path(t_ldsk *young, t_ldsk *old, phydbl sd, phydbl *global_hr,
               new_centr_pos = Uni() * (tree->mmod->lim_up->lonlat[i]-tree->mmod->lim_do->lonlat[i])+tree->mmod->lim_do->lonlat[i];
               
               new_ldsk_pos = Rnorm_Trunc(new_centr_pos,
-                                         sd,
+                                         sd[i],
                                          tree->mmod->lim_do->lonlat[i],
                                          tree->mmod->lim_up->lonlat[i],&err);
               
@@ -273,7 +280,7 @@ void SLFV_Sample_Path(t_ldsk *young, t_ldsk *old, phydbl sd, phydbl *global_hr,
               
               hr -= Log_Dnorm_Trunc(new_ldsk_pos,
                                     new_centr_pos,
-                                    sd,
+                                    sd[i],
                                     tree->mmod->lim_do->lonlat[i],
                                     tree->mmod->lim_up->lonlat[i],&err);
               
@@ -286,7 +293,7 @@ void SLFV_Sample_Path(t_ldsk *young, t_ldsk *old, phydbl sd, phydbl *global_hr,
               
               hr += Log_Dnorm_Trunc(cur_ldsk_pos,
                                     cur_centr_pos,
-                                    sd,
+                                    sd[i],
                                     tree->mmod->lim_do->lonlat[i],
                                     tree->mmod->lim_up->lonlat[i],&err);
               
@@ -301,10 +308,8 @@ void SLFV_Sample_Path(t_ldsk *young, t_ldsk *old, phydbl sd, phydbl *global_hr,
               ldsk->disk->centr->lonlat[i]  = new_centr_pos;                            
             }
           
-          /* new_glnL = SLFV_Lk_Range(young->disk,old->disk,tree); */
-          /* new_glnL = SLFV_Lk_Core_Range(young->disk,old->disk,tree); */
           new_glnL = SLFV_Lk_Range(ldsk->disk,ldsk->prev->disk,tree);
-          /* new_glnL = SLFV_Lk(tree); */
+          if(PHYREX_Total_Number_Of_Intervals(tree) > tree->mmod->max_num_of_intervals) new_glnL = UNLIKELY;
           
           
           /* PhyML_Printf("\n. k: %4d lnL: %f",k,cur_glnL); */
@@ -345,14 +350,16 @@ void SLFV_Sample_Path(t_ldsk *young, t_ldsk *old, phydbl sd, phydbl *global_hr,
 /*////////////////////////////////////////////////////////////
 ////////////////////////////////////////////////////////////*/
 
-t_ldsk *SLFV_Generate_Path(t_ldsk *young, t_ldsk *old, phydbl n_evt, phydbl sd, t_tree *tree)
+t_ldsk *SLFV_Generate_Path(t_ldsk *young, t_ldsk *old, int n_evt, phydbl *sd, t_tree *tree)
 {
   int i,j,swap,err;
-  phydbl *time,dum,mode;
+  phydbl *time,dum,mode,var;
   t_ldsk *path,**ldsk_a;
   t_dsk *disk;
   phydbl X,Y,Xp,Yp;
-  phydbl slope,inter;
+  /* phydbl slope,inter; */
+  phydbl dt_young,dt_old,sum;
+  phydbl pos;
   
   path = NULL;
   
@@ -408,7 +415,6 @@ t_ldsk *SLFV_Generate_Path(t_ldsk *young, t_ldsk *old, phydbl n_evt, phydbl sd,
 
       for(j=0;j<n_evt;j++)
         {
-
           if(j == 0)
             {
               Xp = young->coord->lonlat[i];
@@ -421,32 +427,39 @@ t_ldsk *SLFV_Generate_Path(t_ldsk *young, t_ldsk *old, phydbl n_evt, phydbl sd,
               Yp = ldsk_a[j]->next[0]->disk->time;
             }
           
-          slope = (Yp-Y)/(Xp-X);
-          inter = Y - slope*X;
+          /* slope = (Yp-Y)/(Xp-X); */
+          /* inter = Y - slope*X; */
 
-          if(j == 0)
-            {
-              mode = (young->disk->time - inter)/slope;
-            }
-          else
-            {
-              mode = (ldsk_a[j]->next[0]->disk->time - inter)/slope;
-            }
+          /* if(j == 0) */
+          /*   { */
+          /*     mode = (young->disk->time - inter)/slope; */
+          /*   } */
+          /* else */
+          /*   { */
+          /*     mode = (ldsk_a[j]->next[0]->disk->time - inter)/slope; */
+          /*   } */
 
-          ldsk_a[j]->coord->lonlat[i] = Rnorm_Trunc(mode,
-                                                    sd,
-                                                    tree->mmod->lim_do->lonlat[i],
-                                                    tree->mmod->lim_up->lonlat[i],&err);
+          dt_young = fabs(Yp - ldsk_a[j]->disk->time);
+          dt_old = fabs(Y - ldsk_a[j]->disk->time);
+          sum = dt_young + dt_old;
+          
+          mode = (dt_young/sum)*X + (dt_old/sum)*Xp;
+          var = dt_young*dt_old/sum * sd[i] * sd[i]; 
           
           
-          /* ldsk_a[j]->disk->centr->lonlat[i] = Rnorm_Trunc(mode, */
-          /*                                                 sd, */
-          /*                                                 0.0, */
-          /*                                                 tree->mmod->lim->lonlat[i],&err); */
-
+          /* ldsk_a[j]->coord->lonlat[i] = Rnorm(mode,sqrt(var)); */
+          ldsk_a[j]->coord->lonlat[i]  = Rnorm_Trunc(mode,
+                                                     sqrt(var),
+                                                     tree->mmod->lim_do->lonlat[i],
+                                                     tree->mmod->lim_up->lonlat[i],&err);
           
-          ldsk_a[j]->disk->centr->lonlat[i] = Rnorm_Trunc(ldsk_a[j]->coord->lonlat[i],
-                                                          sd,
+          pos = 0.0;
+          if(ldsk_a[j]->coord->lonlat[i] > tree->mmod->lim_up->lonlat[i]) pos = tree->mmod->lim_up->lonlat[i];
+          else if(ldsk_a[j]->coord->lonlat[i] < tree->mmod->lim_do->lonlat[i]) pos = tree->mmod->lim_do->lonlat[i];
+          else pos = ldsk_a[j]->coord->lonlat[i];
+
+          ldsk_a[j]->disk->centr->lonlat[i] = Rnorm_Trunc(pos,
+                                                          sd[i],
                                                           tree->mmod->lim_do->lonlat[i],
                                                           tree->mmod->lim_up->lonlat[i],&err);
         }
@@ -506,12 +519,12 @@ phydbl SLFV_Sample_Rad_From_Prior(t_tree *tree)
 
 phydbl SLFV_Update_Radius(t_tree *tree)
 {
-  switch(tree->mmod->id)
+  switch(tree->mmod->model_id)
     {
     case SLFV_UNIFORM: { return(-1.0); break;}
     case SLFV_GAUSSIAN:  
       { 
-        return(POW(tree->mmod->sigsq/(SLFV_Rate_Per_Unit_Area(tree)*4.*PI*tree->mmod->mu),0.25));
+        return(POW(tree->mmod->sigsq[0]/(SLFV_Rate_Per_Unit_Area(tree)*4.*PI*tree->mmod->mu),0.25));
         break; 
       }
     }
@@ -531,7 +544,7 @@ phydbl SLFV_Generation_Length(t_tree *tree)
 
 phydbl SLFV_Neighborhood_Size(t_tree *tree)
 {
-  switch(tree->mmod->id)
+  switch(tree->mmod->model_id)
     {
     case SLFV_UNIFORM: { return(1./tree->mmod->mu); break; }
     case SLFV_GAUSSIAN:  { return(2./tree->mmod->mu); break; }
@@ -544,7 +557,7 @@ phydbl SLFV_Neighborhood_Size(t_tree *tree)
 
 phydbl SLFV_Update_Sigsq(t_tree *tree)
 {  
-  switch(tree->mmod->id)
+  switch(tree->mmod->model_id)
     {
     case SLFV_UNIFORM : { return(-1.0); break;}
     case SLFV_GAUSSIAN :  
@@ -633,10 +646,27 @@ phydbl SLFV_Neighborhood_Size_Regression(t_tree *tree)
 
 phydbl SLFV_Lk_Range(t_dsk *young, t_dsk *old, t_tree *tree)
 {
-  if(tree->mmod->id == SLFV_GAUSSIAN) return(SLFV_Lk_Gaussian_Range(young,old,tree));
-  else if(tree->mmod->id == SLFV_UNIFORM) assert(FALSE);
-  else assert(FALSE);
-  return(-1.);
+  phydbl lnP;
+
+  lnP = UNLIKELY;
+  
+  switch(tree->mmod->model_id)
+    {
+    case SLFV_GAUSSIAN :
+      {
+        lnP = SLFV_Lk_Gaussian_Range(young,old,tree);
+        break;
+      }
+    case SLFV_UNIFORM :
+      {
+        PhyML_Fprintf(stderr,"\n. SLFV model with rectangle is not implemented. Sorry...");
+        assert(false);
+        break;
+      }      
+    default : break;
+    }
+
+  return(lnP);
 }
 
 /*////////////////////////////////////////////////////////////
@@ -645,19 +675,15 @@ phydbl SLFV_Lk_Range(t_dsk *young, t_dsk *old, t_tree *tree)
 phydbl SLFV_Lk_Gaussian_Range(t_dsk *young, t_dsk *old, t_tree *tree)
 {
   t_dsk *disk;
-  phydbl lnL,dt;
+  phydbl lnL,dt,disk_lnL;
   int n_evt;
-  
-  /* return(PHYREX_Lk(tree)); */
-  
-  if(PHYREX_Total_Number_Of_Intervals(tree) > tree->mmod->max_num_of_intervals) return UNLIKELY;
-
+    
   assert(young);
 
   lnL  = 0.0;
-
   lnL += SLFV_Lk_Gaussian_Core(young,tree);
 
+  disk_lnL = 0.0;
   dt = 0.0;
   n_evt = 0;
   disk = young->prev;
@@ -671,7 +697,11 @@ phydbl SLFV_Lk_Gaussian_Range(t_dsk *young, t_dsk *old, t_tree *tree)
 
       assert(disk);
       if(disk->time > disk->next->time) return UNLIKELY;
-      lnL += SLFV_Lk_Gaussian_Core(disk,tree);
+      disk_lnL = SLFV_Lk_Gaussian_Core(disk,tree);
+      lnL += disk_lnL;
+
+      if(disk->next != NULL) disk->cum_glnL = disk->next->cum_glnL + disk_lnL;      
+
       if(disk == old) break;
       disk = disk->prev;
     }
@@ -746,7 +776,7 @@ phydbl SLFV_Lk_Gaussian_Core(t_dsk *disk, t_tree *tree)
 
   /* Likelihood for the disk center */
   for(j=0;j<tree->mmod->n_dim;j++) lnL += log(1./(tree->mmod->lim_up->lonlat[j]-tree->mmod->lim_do->lonlat[j]));
-  
+
   return(lnL);
 }
 
@@ -777,6 +807,7 @@ phydbl SLFV_Lk_Gaussian(t_tree *tree)
 
   /* tree->mmod->c_lnL += PHYREX_LnPrior_Radius(tree); */
   
+  // !!!!!!! Really necessary?
   PHYREX_Update_Lindisk_List(tree);
 
   tree->mmod->c_lnL += SLFV_Lk_Gaussian_Core(tree->young_disk,tree);
@@ -802,8 +833,7 @@ phydbl SLFV_Lk_Gaussian(t_tree *tree)
     }
   while(1);
 
-  tree->mmod->c_lnL += PHYREX_Lk_Time_Component(tree);
-
+  /* tree->mmod->c_lnL += TIMES_Lk_SLFV(tree); */
   /* tree->mmod->c_lnL += n_evt * log(tree->mmod->lbda) - tree->mmod->lbda * fabs(tree->young_disk->time - disk->time); */
     
   if(isinf(tree->mmod->c_lnL) || isnan(tree->mmod->c_lnL)) tree->mmod->c_lnL = UNLIKELY;
@@ -929,7 +959,7 @@ t_sarea *SLFV_Simulate_Forward_Core(int n_sites, t_tree *tree)
               lx = disk->ldsk_a[i]->coord->lonlat[0];
               ly = disk->ldsk_a[i]->coord->lonlat[1];
               
-              switch(mmod->id)
+              switch(mmod->model_id)
                 {
                 case SLFV_UNIFORM:
                   {
@@ -977,7 +1007,7 @@ t_sarea *SLFV_Simulate_Forward_Core(int n_sites, t_tree *tree)
               ly = disk->ldsk_a[i]->coord->lonlat[1];
               
               prob_death = 0.0;
-              switch(mmod->id)
+              switch(mmod->model_id)
                 {
                 case SLFV_UNIFORM:
                   {
@@ -1042,7 +1072,7 @@ t_sarea *SLFV_Simulate_Forward_Core(int n_sites, t_tree *tree)
               
               
               /* Generate new location */
-              switch(mmod->id)
+              switch(mmod->model_id)
                 {
                 case SLFV_UNIFORM: { PHYREX_Runif_Rectangle_Overlap(new_ldsk,new_disk,mmod); break; }
                 case SLFV_GAUSSIAN:  { PHYREX_Rnorm_Trunc(new_ldsk,new_disk,mmod); break; }
@@ -1358,7 +1388,7 @@ t_sarea *SLFV_Simulate_Forward_Core(int n_sites, t_tree *tree)
       if((disk->prev->ldsk != NULL) && (disk->prev->ldsk->n_next == 0))
         {
           /* printf("\n. free %s @ %f",disk->ldsk->coord->id,disk->time); */
-          Free_Ldisk(disk->prev->ldsk);
+          PHYREX_Free_Ldisk(disk->prev->ldsk);
           disk->prev->ldsk = NULL;
         }
 
@@ -1433,7 +1463,7 @@ phydbl SLFV_Simulate_Backward_Core(t_dsk *init_disk, int avoid_multiple_mergers,
   oldest_disk = disk;
   reached_oldest_disk = NO;
   if(init_disk->prev == NULL) reached_oldest_disk = YES; // Only one sampled disk
-  
+
   lnL = 0.0;
   disk = init_disk;
   do
@@ -1485,27 +1515,42 @@ phydbl SLFV_Simulate_Backward_Core(t_dsk *init_disk, int avoid_multiple_mergers,
           new_disk->time = new_time;
           
           /* Coordinates of new event */
-          for(j=0;j<mmod->n_dim;++j) new_disk->centr->lonlat[j] = Uni()*(mmod->lim_up->lonlat[j]-mmod->lim_do->lonlat[j])+mmod->lim_do->lonlat[j];
-
+          for(j=0;j<mmod->n_dim;++j)
+            {
+              switch(mmod->model_id)
+                {
+                case SLFV_UNIFORM : case SLFV_GAUSSIAN : 
+                  { 
+                    new_disk->centr->lonlat[j] = Uni()*(mmod->lim_up->lonlat[j]-mmod->lim_do->lonlat[j])+mmod->lim_do->lonlat[j];
+                    break; 
+                  }
+                case RW : case RRW_GAMMA : case RRW_LOGNORMAL :  
+                  { 
+                    new_disk->centr->lonlat[j] = disk->ldsk_a[Rand_Int(0,disk->n_ldsk_a-1)]->coord->lonlat[j];
+                    break; 
+                  }
+                default : break;
+                }
+            }
+          
           for(j=0;j<mmod->n_dim;++j) lnL += log(1./(mmod->lim_up->lonlat[j]-mmod->lim_do->lonlat[j]));
-
           
           /* Populate new_disk->ldsk_a array */
-          PHYREX_Update_Lindisk_List_Core(new_disk,tree);
-          
+          PHYREX_Update_Lindisk_List_Core(new_disk,tree);          
+
           new_disk->ldsk = NULL;
           /* Which lineages in new_disk->ldsk_a are hit? */
           for(i=0;i<new_disk->n_ldsk_a;++i)
             {
               prob_hit = -1.;
-              switch(mmod->id)
+              switch(mmod->model_id)
                 {
                 case SLFV_UNIFORM : 
                   { 
                     prob_hit = mmod->mu; 
                     break; 
                   }
-                case SLFV_GAUSSIAN : case RW : case RRW :  
+                case SLFV_GAUSSIAN :   
                   { 
                     prob_hit = log(mmod->mu);
                     for(j=0;j<mmod->n_dim;++j)
@@ -1514,9 +1559,13 @@ phydbl SLFV_Simulate_Backward_Core(t_dsk *init_disk, int avoid_multiple_mergers,
                           -pow(new_disk->ldsk_a[i]->coord->lonlat[j] -
                                new_disk->centr->lonlat[j],2)/(2.*pow(mmod->rad,2));
                       }
-                    prob_hit = exp(prob_hit);                    
+                    prob_hit = exp(prob_hit);
                     break; 
                   }
+                  case RW : case RRW_GAMMA : case RRW_LOGNORMAL :
+                    {
+                      prob_hit = 1./(new_disk->n_ldsk_a+1.);
+                    }
                 }
 
               
@@ -1524,6 +1573,7 @@ phydbl SLFV_Simulate_Backward_Core(t_dsk *init_disk, int avoid_multiple_mergers,
               if(!(u > prob_hit)) // disk->ldsk_a[i] is  hit
                 {
                   lnL += log(prob_hit); 
+
                   // new_disk->ldsk_a[i] is hit -> coalesce (or just jump) to parent (i.e., new_disk->ldsk)
                   if(new_disk->ldsk == NULL)
                     {
@@ -1531,14 +1581,14 @@ phydbl SLFV_Simulate_Backward_Core(t_dsk *init_disk, int avoid_multiple_mergers,
                       PHYREX_Init_Lindisk_Node(new_disk->ldsk,new_disk,mmod->n_dim);
                       
                       // Generate coordinates for newly created ldsk
-                      switch(tree->mmod->id)
+                      switch(tree->mmod->model_id)
                         {
                         case SLFV_UNIFORM :
                           {
                             PHYREX_Runif_Rectangle_Overlap(new_disk->ldsk,new_disk,tree->mmod);
                             break;
                           }
-                        case SLFV_GAUSSIAN : case RW : case RRW :
+                        case SLFV_GAUSSIAN : case RW : case RRW_GAMMA : case RRW_LOGNORMAL :
                           {
                             PHYREX_Rnorm_Trunc(new_disk->ldsk,new_disk,tree->mmod);
                             for(j=0;j<tree->mmod->n_dim;++j)
@@ -1559,6 +1609,7 @@ phydbl SLFV_Simulate_Backward_Core(t_dsk *init_disk, int avoid_multiple_mergers,
                 {
                   lnL += log(1. - prob_hit);
                 }
+              
               if(new_disk->ldsk && new_disk->ldsk->n_next == 2 && avoid_multiple_mergers == YES) break;
             }
         }
@@ -1649,9 +1700,7 @@ t_tree *SLFV_Simulate(int n_otu, int n_sites, phydbl w, phydbl h, phydbl  lbda,
   if(mmod->mu < mmod->min_mu) mmod->mu = mmod->min_mu;
 
 
-  mmod->sigsq = SLFV_Update_Sigsq(tree);
-
-
+  mmod->sigsq[0] = SLFV_Update_Sigsq(tree);
   
   // Duration of a generation in number of events
   mmod->gen_cal_time = 1./(2.*mmod->mu*pow(mmod->rad,2)/pow(w*h,2)*
@@ -1705,8 +1754,8 @@ t_tree *SLFV_Simulate(int n_otu, int n_sites, phydbl w, phydbl h, phydbl  lbda,
   SLFV_Simulate_Backward_Core(tree->young_disk,NO,tree);
 
   PHYREX_Ldsk_To_Tree(tree);
-  Update_Ancestors(tree->n_root,tree->n_root->v[2],tree);
-  Update_Ancestors(tree->n_root,tree->n_root->v[1],tree);
+  Update_Ancestors(tree->n_root,tree->n_root->v[2],tree->n_root->b[2],tree);
+  Update_Ancestors(tree->n_root,tree->n_root->v[1],tree->n_root->b[1],tree);
   RATES_Fill_Lca_Table(tree);
   
   
@@ -1717,12 +1766,13 @@ t_tree *SLFV_Simulate(int n_otu, int n_sites, phydbl w, phydbl h, phydbl  lbda,
 
   tree->rates->bl_from_rt = YES;
   tree->rates->clock_r    = 0.1/fabs(2.*T);
-  tree->rates->model      = STRICTCLOCK;
+  tree->rates->model_id      = STRICTCLOCK;
 
-  RATES_Update_Cur_Bl(tree);
+  RATES_Update_Edge_Lengths(tree);
 
   Init_Model(cdata,mod,io);
-
+  Set_Model_Parameters(mod);
+  
   tree->mod->ras->n_catg   = 1;
   tree->mod->whichmodel    = HKY85;
   tree->mod->kappa->v      = 4.0;
@@ -1761,7 +1811,7 @@ t_tree *SLFV_Simulate(int n_otu, int n_sites, phydbl w, phydbl h, phydbl  lbda,
                mmod->mu,
                mmod->rad,
                tree->rates->clock_r,
-               mmod->sigsq);
+               mmod->sigsq[0]);
 
   PhyML_Printf("\n. Useful statistics: t.root=%f n.int=%d n.coal=%d n.hit=%d root.x=%f root.y=%f nt.div=%f\n",
                disk->time,
@@ -1854,7 +1904,7 @@ t_tree *SLFV_Simulate_Independent_Loci(int n_otu, int n_loci, phydbl w, phydbl h
   Random_Tree(tree);
   Connect_CSeqs_To_Nodes(cdata,io,tree);
   tree->rates = RATES_Make_Rate_Struct(tree->n_otu);
-  io->rates->model = STRICTCLOCK;
+  io->rates->model_id = STRICTCLOCK;
   RATES_Init_Rate_Struct(tree->rates,io->rates,tree->n_otu);
 
   tree->times = TIMES_Make_Time_Struct(tree->n_otu);
@@ -1910,7 +1960,7 @@ t_tree *SLFV_Simulate_Independent_Loci(int n_otu, int n_loci, phydbl w, phydbl h
   mmod->gen_cal_time *= 1./mmod->lbda;
 
   /* Dispersal parameter (per generation) */
-  mmod->sigsq = 4.*pow(mmod->rad,4)*mmod->lbda*PI*mmod->mu/area * mmod->gen_cal_time;
+  mmod->sigsq[0] = 4.*pow(mmod->rad,4)*mmod->lbda*PI*mmod->mu/area * mmod->gen_cal_time;
 
  
   phydbl p = SLFV_Prob_Two_Random_Lineages_Coal_One_Event(w,h,mmod->mu,mmod->rad);
@@ -1923,7 +1973,7 @@ t_tree *SLFV_Simulate_Independent_Loci(int n_otu, int n_loci, phydbl w, phydbl h
                mmod->mu,
                mmod->rad,
                tree->rates->clock_r,
-               mmod->sigsq,
+               mmod->sigsq[0],
                2./mmod->mu,
                (1./(1.-exp(-4.*pow(mmod->mu,2)*pow(PI,2)*pow(mmod->rad,4)/(pow(w*h,2))*mmod->lbda*mmod->gen_cal_time)))/area,
                rhoe,
@@ -1999,8 +2049,8 @@ t_tree *SLFV_Simulate_Independent_Loci(int n_otu, int n_loci, phydbl w, phydbl h
       PHYREX_Ldsk_To_Tree(tree);  
 
       
-      Update_Ancestors(tree->n_root,tree->n_root->v[2],tree);
-      Update_Ancestors(tree->n_root,tree->n_root->v[1],tree);
+      Update_Ancestors(tree->n_root,tree->n_root->v[2],tree->n_root->b[2],tree);
+      Update_Ancestors(tree->n_root,tree->n_root->v[1],tree->n_root->b[1],tree);
       RATES_Fill_Lca_Table(tree);
 
 
@@ -2016,7 +2066,7 @@ t_tree *SLFV_Simulate_Independent_Loci(int n_otu, int n_loci, phydbl w, phydbl h
       PhyML_Printf("\n. #!# mutation rate at that locus: %G subst. per time unit",tree->rates->clock_r);
       subst_rate += tree->rates->clock_r;
       
-      RATES_Update_Cur_Bl(tree);
+      RATES_Update_Edge_Lengths(tree);
 
       char *s = Write_Tree(tree);
       PhyML_Printf("\n. #@# tree: %s",s);
@@ -2028,8 +2078,8 @@ t_tree *SLFV_Simulate_Independent_Loci(int n_otu, int n_loci, phydbl w, phydbl h
       while(disk->prev)
         {
           disk = disk->prev;
-          if(disk->next->ldsk != NULL) Free_Ldisk(disk->next->ldsk);
-          Free_Disk(disk->next);
+          if(disk->next->ldsk != NULL) PHYREX_Free_Ldisk(disk->next->ldsk);
+          PHYREX_Free_Disk(disk->next);
         }
 
       locus_idx++;
@@ -2160,3 +2210,39 @@ void SLFV_Sum_Coal_Rate(t_ldsk *l0, t_ldsk *l1, int n, t_tree *tree)
   PhyML_Printf("\n");
   Exit("\n");
 }
+
+/*////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////*/
+
+void SLFV_Generate_Ldsk_New_Location(t_ldsk *l, t_ldsk *prev_l, phydbl rad, phydbl *hr, int dim_idx, t_tree *tree)
+{
+  int err;
+  phydbl new_c,new_l,c;
+
+  err = NO;
+
+  /* Forward move */
+  new_c = Uni()*(tree->mmod->lim_up->lonlat[dim_idx]-tree->mmod->lim_do->lonlat[dim_idx])+tree->mmod->lim_do->lonlat[dim_idx];
+  (*hr) -= log(1./(tree->mmod->lim_up->lonlat[dim_idx]-tree->mmod->lim_do->lonlat[dim_idx]));
+  c = new_c;
+  new_l = Rnorm_Trunc(c,rad,tree->mmod->lim_do->lonlat[dim_idx],tree->mmod->lim_up->lonlat[dim_idx],&err);
+  (*hr) -= Log_Dnorm_Trunc(new_l,c,rad,tree->mmod->lim_do->lonlat[dim_idx],tree->mmod->lim_up->lonlat[dim_idx],&err);
+  
+  /* Reverse move */
+  c = prev_l->disk->centr->lonlat[dim_idx];
+  (*hr) += Log_Dnorm_Trunc(prev_l->coord->lonlat[dim_idx],c,rad,tree->mmod->lim_do->lonlat[dim_idx],tree->mmod->lim_up->lonlat[dim_idx],&err);
+  
+  l->disk->centr->lonlat[dim_idx] = new_c;
+  l->coord->lonlat[dim_idx] = new_l;
+  
+  assert(err == NO);
+}
+
+/*////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////*/
+/*////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////*/
+/*////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////*/
+/*////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////*/
