@@ -128,8 +128,6 @@ int Spr(phydbl init_lnL, phydbl prop_spr, t_tree *tree)
   Set_Both_Sides(YES,tree);
   Lk(NULL,tree);
   tree->best_lnL = tree->c_lnL;
-
-  /* PhyML_Printf("\n. init: %f",tree->c_lnL); */
   
   for(i=0;i<MAX(1,(int)((2*tree->n_otu-3)*prop_spr));++i)
     {
@@ -145,7 +143,6 @@ int Spr(phydbl init_lnL, phydbl prop_spr, t_tree *tree)
           Spr_Subtree(b,b->rght,tree);
         }
     }
-
   
   Free(br_idx);
     
@@ -750,7 +747,6 @@ void Global_Spr_Search(t_tree *tree)
   if(tree->io->print_json_trace == YES) JSON_Tree_Io(tree,tree->io->fp_out_json_trace);
 
   if(tree->verbose > VL0 && tree->io->quiet == NO) PhyML_Printf("\n\n. Score of initial tree: %.2f",tree->c_lnL);
-
                  
   tree->mod->s_opt->min_diff_lk_move  = 1.E-1;
   tree->mod->s_opt->min_diff_lk_local = 1.E-1;
@@ -781,7 +777,6 @@ void Global_Spr_Search(t_tree *tree)
   iter = 0;
   do
     {
-      
       if(!(iter%freq))
         {
           for(int i=0;i<2*tree->n_otu-3;++i) MIXT_Multiply_Scalar_Dbl(tree->a_edges[i]->l,Rgamma((phydbl)(tune_l_mult*iter+1),(phydbl)(1./(tune_l_mult*iter+1))));
@@ -864,8 +859,6 @@ void Global_Spr_Search(t_tree *tree)
   tree->mod->s_opt->spr_pars                  = NO;
   tree->mod->s_opt->min_diff_lk_move          = 1.E-2;
   tree->mod->s_opt->min_diff_lk_local         = 1.E-2;
-  tree->mod->s_opt->apply_spr_right_away      = YES;
-  tree->mod->s_opt->apply_spr                 = YES;
   tree->mod->s_opt->eval_list_regraft         = NO;
   tree->mod->s_opt->max_delta_lnL_spr_current = 0.0;
   tree->mod->s_opt->min_n_triple_moves        = 1;
@@ -881,7 +874,7 @@ void Global_Spr_Search(t_tree *tree)
         {
           for(int i=0;i<2*tree->n_otu-3;++i) MIXT_Multiply_Scalar_Dbl(tree->a_edges[i]->l,Rgamma((phydbl)(tune_l_mult*iter+1),(phydbl)(1./(tune_l_mult*iter+1))));
           for(int i=0;i<2*tree->n_otu-3;++i) Set_Scalar_Dbl_Min_Thresh(tree->mod->l_min,tree->a_edges[i]->l);
-          for(int i=0;i<2*tree->n_otu-3;++i) Set_Scalar_Dbl_Max_Thresh(tree->mod->l_max,tree->a_edges[i]->l);          
+          for(int i=0;i<2*tree->n_otu-3;++i) Set_Scalar_Dbl_Max_Thresh(tree->mod->l_max,tree->a_edges[i]->l);
           freq--;
           if(freq < 1) freq=1;
         }
@@ -890,7 +883,7 @@ void Global_Spr_Search(t_tree *tree)
       Optimize_Br_Len_Serie(2,tree);
       
       if(!(iter%round_freq) && iter > 0) Round_Optimize(tree,1000);
-
+      
       if(tree->verbose > VL0 && tree->io->quiet == NO)
         {
           time(&t_cur);
@@ -952,8 +945,6 @@ void Global_Spr_Search(t_tree *tree)
   tree->mod->s_opt->l_min_spr                 = 1.E-4;
   tree->mod->s_opt->min_diff_lk_move          = 1.E-2;
   tree->mod->s_opt->min_diff_lk_local         = 1.E-2;
-  tree->mod->s_opt->apply_spr_right_away      = YES;
-  tree->mod->s_opt->apply_spr                 = YES;
   tree->mod->s_opt->eval_list_regraft         = YES;
   tree->mod->s_opt->max_delta_lnL_spr_current = 0.0;
   tree->mod->s_opt->min_n_triple_moves        = 5;
@@ -964,7 +955,7 @@ void Global_Spr_Search(t_tree *tree)
     {
       Spr(tree->c_lnL,1.0,tree);
       Optimize_Br_Len_Serie(2,tree);
-      
+            
       if(!(iter%round_freq)) Round_Optimize(tree,1000);
 
       if(tree->verbose > VL0 && tree->io->quiet == NO)
@@ -2090,257 +2081,8 @@ void Spr_Random_Explore(t_tree *tree, phydbl anneal_temp, phydbl prop_spr, int d
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-
-void Prune_Regraft_Time_Tree(t_tree *tree)
-{
-  phydbl u,ratio;
-  phydbl t_min,t_max;
-  phydbl cur_lnL_seq,new_lnL_seq;
-  phydbl cur_lnL_time,new_lnL_time;
-  phydbl new_t;
-  int i,j,k,prune_idx,n_regraft_nd,regraft_idx,dir_prune;
-  phydbl *times;
-  int rnd_dir,dir_v1,dir_v2,keepon;
-  t_node *prune,*prune_daughter,*new_regraft_nd,*cur_regraft_nd;
-  t_ll *regraft_nd_list;
-  t_edge *target, *ori_target, *residual,*regraft_edge;
-  phydbl regraft_t_min,regraft_t_max;
   
-  times = tree->times->nd_t;
-  
-  do
-    {
-      keepon = NO;
-      for(i=tree->n_otu;i<2*tree->n_otu-2;++i) // for each internal node
-        {
-          
-          TIMES_Update_Node_Ordering(tree);
-          
-          RATES_Record_Times(tree);
-          
-          cur_lnL_seq  = tree->c_lnL;
-          new_lnL_seq  = UNLIKELY;
-          cur_lnL_time = tree->times->c_lnL_times;
-          new_lnL_time = UNLIKELY;
-          
-          regraft_edge   = NULL;
-          new_regraft_nd = NULL;
-          cur_regraft_nd = NULL;
-          new_t          = 0.0;
-          
-          // Prune node
-          prune_idx = i;      
-          prune = tree->a_nodes[prune_idx];      
-          assert(prune && prune->tax == NO);
-          
-          
-          // Select a daughter of prune node
-          dir_v1 = dir_v2 = -1;
-          for(j=0;j<3;++j) 
-            if(prune->v[j] != prune->anc && prune->b[j] != tree->e_root)
-              {
-                if(dir_v1 < 0) dir_v1 = j;
-                else           dir_v2 = j;
-              }
-          
-          u = Uni();
-          if(u < 0.5) rnd_dir = dir_v1;
-          else        rnd_dir = dir_v2;
-          
-          prune_daughter = prune->v[rnd_dir];
-          cur_regraft_nd = prune->v[rnd_dir == dir_v1 ? dir_v2 : dir_v1];            
-          
-          if(prune == tree->n_root)
-            {
-              if(prune_daughter == prune->v[dir_v1] && prune->v[dir_v2]->tax == YES)
-                {
-                  prune_daughter = prune->v[dir_v2];
-                  cur_regraft_nd = prune->v[dir_v1];
-                }
-              
-              if(prune_daughter == prune->v[dir_v2] && prune->v[dir_v1]->tax == YES)
-                {
-                  prune_daughter = prune->v[dir_v1];
-                  cur_regraft_nd = prune->v[dir_v2];
-                }
-            }
-          
-          assert(prune_daughter->anc == prune);
-          
-          dir_prune = -1;
-          for(j=0;j<3;j++)
-            {
-              if(prune_daughter->v[j] == prune || prune_daughter->b[j] == tree->e_root)
-                {
-                  dir_prune = j;
-                  break;
-                }
-            }
-          assert(dir_prune > -1);
-          
-          
-          // Get the list of potential regraft nodes (oldest node on regraft edge)
-          regraft_nd_list = DATE_List_Of_Regraft_Nodes(prune_daughter->v[dir_prune],prune_daughter,&regraft_t_min,&regraft_t_max,NO,tree);
-          assert(regraft_nd_list);
-          if(prune == tree->n_root) Push_Bottom_Linked_List(prune,&regraft_nd_list,YES);
-          
-          
-          // Number of regraft nodes
-          n_regraft_nd = Linked_List_Len(regraft_nd_list);
-          
-          
-          for(j=0;j<n_regraft_nd;j++)
-            {
-              // Randomly select one (uniform)
-              regraft_idx = Rand_Int(0,n_regraft_nd-1);      
-              new_regraft_nd = Linked_List_Elem(regraft_idx,regraft_nd_list);      
-              
-              // Time of regraft node           
-              t_max = MIN(times[prune_daughter->num],times[new_regraft_nd->num]);      
-              if(new_regraft_nd == tree->n_root) t_min = 10.0*t_max;
-              else t_min = times[new_regraft_nd->anc->num];
-              t_min = MAX(t_min,regraft_t_min);
-              
-              new_t = Uni()*(t_max-t_min) + t_min;      
-              
-              
-              // New age
-              if(prune == tree->n_root || new_regraft_nd == tree->n_root)
-                {
-                  if(prune == tree->n_root)
-                    {
-                      if(prune_daughter == tree->n_root->v[1]) times[tree->n_root->num] = times[tree->n_root->v[2]->num];
-                      else                                     times[tree->n_root->num] = times[tree->n_root->v[1]->num];
-                      times[prune_daughter->v[dir_prune]->num] = new_t;
-                    }
-                  if(new_regraft_nd == tree->n_root)
-                    {
-                      times[prune_daughter->v[dir_prune]->num] = times[tree->n_root->num];
-                      times[tree->n_root->num] = new_t;
-                    }
-                }
-              else
-                {
-                  times[prune->num] = new_t;
-                }
-              
-              
-              // Prune
-              target = residual = NULL;
-              Prune_Subtree(prune_daughter->v[dir_prune],
-                            prune_daughter,
-                            &target,&residual,tree);
-              ori_target = target;
-              
-              
-              // Regraft edge is the one sitting above regraft_nd
-              if(new_regraft_nd == tree->n_root->v[1] ||
-                 new_regraft_nd == tree->n_root->v[2] ||
-                 new_regraft_nd == tree->n_root) regraft_edge = tree->e_root;
-              else
-                {
-                  for(k=0;k<3;k++) if(new_regraft_nd->v[k] == new_regraft_nd->anc) break;
-                  assert(k!=3);
-                  regraft_edge = new_regraft_nd->b[k];
-                }
-              
-              assert(regraft_edge);      
-              
-              
-              // Regraft
-              Graft_Subtree(regraft_edge,
-                            prune_daughter->v[dir_prune],
-                            prune_daughter,
-                            residual,
-                            new_regraft_nd,tree);
-              
-              
-              if(!TIMES_Check_Node_Height_Ordering(tree))
-                {
-                  PhyML_Fprintf(stderr,"\n== prune[%d]->t:%.3f daughter[%d]->t:%.3f prune_anc[%d]->t:%.3f regraft[%d]->t:%.3f regraft_anc[%d]->t:%.3f [effective:%d] t_prior_min/max: [prune:[%.3f %.3f] regraft:[%.3f %.3f]] ",
-                                prune->num,
-                                times[prune->num],
-                                prune_daughter->num,
-                                times[prune_daughter->num],
-                                prune->anc ? prune->anc->num : -1,
-                                prune->anc ? times[prune->anc->num] : -1.,
-                                new_regraft_nd->num,
-                                times[new_regraft_nd->num],
-                                new_regraft_nd->anc ? new_regraft_nd->anc->num : -1,
-                                new_regraft_nd->anc ? times[new_regraft_nd->anc->num] : +1.,
-                                prune->num,
-                                tree->times->t_prior_min[prune->num],
-                                tree->times->t_prior_max[prune->num],
-                                tree->times->t_prior_min[new_regraft_nd->num],
-                                tree->times->t_prior_max[new_regraft_nd->num]);
-                  PhyML_Fprintf(stderr,"\n== root: %d %d %d",tree->n_root->num,tree->n_root->v[1]->num,tree->n_root->v[2]->num);
-                  Generic_Exit(__FILE__,__LINE__,__FUNCTION__);    
-                }
-              
-              DATE_Assign_Primary_Calibration(tree);
-              new_lnL_time = TIMES_Lk_Times(NO,tree); 
-              
-              if(new_lnL_time > UNLIKELY)
-                {
-                  Set_Both_Sides(NO,tree);
-                  new_lnL_seq = Lk(NULL,tree);
-                }
-              
-              ratio = (new_lnL_seq - cur_lnL_seq);
-
-              
-              if(ratio < .0)
-                {
-                  // Reject
-                  Prune_Subtree(prune_daughter->v[dir_prune],
-                                prune_daughter,
-                                &target,&residual,tree);
-                  Graft_Subtree(ori_target,
-                                prune_daughter->v[dir_prune],
-                                prune_daughter,residual,prune == tree->n_root ? tree->n_root : cur_regraft_nd,tree);
-                  
-                  RATES_Reset_Times(tree);
-                  RATES_Update_Cur_Bl(tree);
-                  DATE_Assign_Primary_Calibration(tree);
-                  TIMES_Lk_Times(NO,tree); 
-                  
-                  
-                  if(!(tree->times->c_lnL_times > UNLIKELY))
-                    {
-                      printf("\n== time prune: %f",times[prune->num]);
-                      printf("\n== time prune_daughter: %f",times[prune_daughter->num]);
-                      printf("\n== prune: %d prune_daughter: %d prune_daughter->v[dir_prune]: %d cur_regraft_nd: %d new_regraft_nd: %d",
-                             prune->num,
-                             prune_daughter->num,
-                             prune_daughter->v[dir_prune]->num,
-                             cur_regraft_nd->num,
-                             new_regraft_nd->num);
-                      TIMES_Lk_Times(YES,tree); 
-                      fflush(NULL);
-                    }
-                  assert(tree->times->c_lnL_times > UNLIKELY);
-                  
-                  tree->c_lnL              = cur_lnL_seq;
-                  tree->times->c_lnL_times = cur_lnL_time;
-                }
-              else
-                {
-                  PhyML_Printf("\n. Hill-climbing step :: subtree [%4d/%4d] target [%4d/%4d] lnl: %f delta: %f",
-                               i,2*tree->n_otu-2,
-                               j,n_regraft_nd,
-                               tree->c_lnL,
-                               ratio);
-                  if(ratio > 10.) keepon = YES;
-                  break;
-                }
-            }
-          Free_Linked_List(regraft_nd_list);
-        }
-    }while(keepon == YES);
-}
-  
-  
-  //////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
