@@ -22,7 +22,7 @@ the GNU public licence.  See http://www.opensource.org for details.
 #include "times.h"
 #include "mixt.h"
 
-#if defined(__SSE3__)
+#if (defined(__SSE__) || defined(__SSE2__) || defined(__SSE3__))
 
 void SSE_Update_Partial_Lk(t_tree *tree,t_edge *b_fcus,t_node *n);
 void SSE_Update_Eigen_Lr(t_edge *b, t_tree *tree);
