@@ -53,6 +53,12 @@ extern int n_sec2;
 
 #define __FUNCTION__ NULL
 
+#define EMPIRICAL 0
+#define ML 1
+#define USER 2
+#define MODEL 3
+
+
 #define For(i,n)                     for(i=0; i<n; i++)
 #define Fors(i,n,s)                  for(i=0; i<n; i+=s)
 #define PointGamma(prob,alpha,beta)  PointChi2(prob,2.0*(alpha))/(2.0*(beta))
@@ -152,12 +158,11 @@ static inline int isinf_ld (long double x) { return isnan (x - x); }
 #define  R_MAT_WEIGHT_MIN 0.01
 #define  R_MAT_WEIGHT_MAX 100.
 
-#define  E_FRQ_MIN 0.01
-#define  E_FRQ_MAX 0.99
-
-#define  UNSCALED_E_FRQ_MIN -1000.
-#define  UNSCALED_E_FRQ_MAX +10.
+#define  E_FRQ_MIN 0.001
+#define  E_FRQ_MAX 0.999
 
+#define  UNSCALED_E_FRQ_MIN -100.
+#define  UNSCALED_E_FRQ_MAX +100.
 
 #define  TSTV_MIN 0.05
 #define  TSTV_MAX 100.0
@@ -165,18 +170,32 @@ static inline int isinf_ld (long double x) { return isnan (x - x); }
 #define  PINV_MIN 0.00001
 #define  PINV_MAX 0.99999
 
-#define RR_MIN 0.01
-#define RR_MAX 100.0
+#define RR_MIN 0.0001
+#define RR_MAX 10000.0
 
 #define UNSCALED_RR_MIN log(RR_MIN)
 #define UNSCALED_RR_MAX log(RR_MAX)
 
 
+#ifdef PHYML
+#define GAMMA_RR_UNSCALED_MIN -1000.
+#define GAMMA_RR_UNSCALED_MAX 1000.
+/* #define GAMMA_RR_UNSCALED_MIN 0.01 */
+/* #define GAMMA_RR_UNSCALED_MAX 100. */
+#else
 #define GAMMA_RR_UNSCALED_MIN 0.01
 #define GAMMA_RR_UNSCALED_MAX 200.
+#endif
 
+
+#ifdef PHYML
+#define GAMMA_R_PROBA_UNSCALED_MIN -5.
+#define GAMMA_R_PROBA_UNSCALED_MAX 5.
+#else
 #define GAMMA_R_PROBA_UNSCALED_MIN 0.01
 #define GAMMA_R_PROBA_UNSCALED_MAX 200.
+#endif
+
 
 #define PHYREX_UNIFORM 0
 #define PHYREX_NORMAL  1
@@ -362,6 +381,7 @@ static inline int isinf_ld (long double x) { return isnan (x - x); }
 #define THORNE         4
 #define GUINDON        5
 #define STRICTCLOCK    6
+#define GAMMA          7
 #define NONE          -1
 
 #define ALRTSTAT       1
@@ -1103,8 +1123,6 @@ typedef struct __EquFreq {
 
   phydbl                  *b_frq; /*! vector of empirical state frequencies */
   
-  short int empirical_state_freq;
-  short int      user_state_freq;
   vect_dbl          *user_b_freq; /*! user-defined nucleotide frequencies */
 
   struct __EquFreq         *next;
@@ -1353,7 +1371,7 @@ typedef struct __Optimiz { /*! parameters to be optimised (mostly used in 'optim
   short int                opt_kappa; /*! =1 -> the ts/tv ratio parameter is optimised */
   short int               opt_lambda; /*! =1 -> the F84|TN93 model specific parameter is optimised */
   short int               opt_pinvar; /*! =1 -> the proportion of invariants is optimised */
-  short int           opt_state_freq; /*! =1 -> the nucleotide frequencies are optimised */
+  short int               state_freq; 
   short int                   opt_rr; /*! =1 -> the relative rate parameters of the GTR or the customn model are optimised */
   short int          opt_subst_param; /*! if opt_topo=0 and opt_subst_param=1 -> the numerical parameters of the
                    model are optimised. if opt_topo=0 and opt_free_param=0 -> no parameter is
@@ -1568,6 +1586,7 @@ typedef struct __T_Rate {
   phydbl nu; /*! Parameter of the Exponential distribution for the corresponding model */
   phydbl min_nu;
   phydbl max_nu;
+  phydbl autocor_rate_prior;
   phydbl covdet;
   phydbl sum_invalid_areas;
 
@@ -2014,6 +2033,8 @@ typedef struct __Migrep_Model{
   int           max_num_of_intervals;
   int                     update_rad;
 
+  short int integrateAncestralLocations;
+  
   phydbl                        lbda; // rate at which events occur
   phydbl                    min_lbda; // min of rate at which events occur
   phydbl                    max_lbda; // max of rate at which events occur
@@ -2057,6 +2078,8 @@ typedef struct __Migrep_Model{
 
   phydbl             disp_prior_mean;
 
+  phydbl            rrw_param_val; // value of parameter governing the variance of diffusion coefficient across edges  
+  
 }t_phyrex_mod;
 
 /*!********************************************************/
@@ -2231,13 +2254,13 @@ void Br_Len_Involving_Invar(t_tree *tree);
 void Br_Len_Not_Involving_Invar(t_tree *tree);
 void Getstring_Stdin(char *s);
 phydbl Num_Derivatives_One_Param(phydbl (*func)(t_tree *tree), t_tree *tree,
-                                 phydbl f0, phydbl *param, int which, int n_param, phydbl stepsize, int logt,
+                                 phydbl f0, phydbl *param, int which, int n_param, phydbl stepsize, short int logt, short int expt,
                                  phydbl *err, int precise, int is_positive);
 phydbl Num_Derivatives_One_Param_Nonaligned(phydbl (*func)(t_tree *tree), t_tree *tree,
-                                            phydbl f0, phydbl **param, int which, int n_param, phydbl stepsize, int logt,
+                                            phydbl f0, phydbl **param, int which, int n_param, phydbl stepsize, short int logt, short int expt,
                                             phydbl *err, int precise, int is_positive);
-int Num_Derivative_Several_Param(t_tree *tree,phydbl *param,int n_param,phydbl stepsize,int logt,phydbl(*func)(t_tree *tree),phydbl *derivatives, int is_positive);
-int Num_Derivative_Several_Param_Nonaligned(t_tree *tree, phydbl **param, int n_param, phydbl stepsize, int logt,
+int Num_Derivative_Several_Param(t_tree *tree, phydbl *param, int n_param, phydbl stepsize, short int logt, short int expt, phydbl(*func)(t_tree *tree),phydbl *derivatives, int is_positive);
+int Num_Derivative_Several_Param_Nonaligned(t_tree *tree, phydbl **param, int n_param, phydbl stepsize, short int logt, short int expt,
                                             phydbl (*func)(t_tree *tree), phydbl *derivatives, int is_positive);
 int Compare_Two_States(char *state1,char *state2,int state_size);
 void Copy_One_State(char *from,char *to,int state_size);
